# collibra-importer
<p>The Import API is an efficient way to load large volumes of data into the Collibra Data Governance Center. The API can automatically differentiate between creating and updating data.</p>

This Python package is automatically generated by the [OpenAPI Generator](https://openapi-generator.tech) project:

- API version: 2.0
- Package version: 1.0.0
- Build package: org.openapitools.codegen.languages.PythonClientCodegen

## Requirements.

Python &gt;&#x3D;3.7

## Migration from other generators like python and python-legacy

### Changes
1. This generator uses spec case for all (object) property names and parameter names.
    - So if the spec has a property name like camelCase, it will use camelCase rather than camel_case
    - So you will need to update how you input and read properties to use spec case
2. Endpoint parameters are stored in dictionaries to prevent collisions (explanation below)
    - So you will need to update how you pass data in to endpoints
3. Endpoint responses now include the original response, the deserialized response body, and (todo)the deserialized headers
    - So you will need to update your code to use response.body to access deserialized data
4. All validated data is instantiated in an instance that subclasses all validated Schema classes and Decimal/str/list/tuple/frozendict/NoneClass/BoolClass/bytes/io.FileIO
    - This means that you can use isinstance to check if a payload validated against a schema class
    - This means that no data will be of type None/True/False
        - ingested None will subclass NoneClass
        - ingested True will subclass BoolClass
        - ingested False will subclass BoolClass
        - So if you need to check is True/False/None, instead use instance.is_true_oapg()/.is_false_oapg()/.is_none_oapg()
5. All validated class instances are immutable except for ones based on io.File
    - This is because if properties were changed after validation, that validation would no longer apply
    - So no changing values or property values after a class has been instantiated
6. String + Number types with formats
    - String type data is stored as a string and if you need to access types based on its format like date,
    date-time, uuid, number etc then you will need to use accessor functions on the instance
    - type string + format: See .as_date_oapg, .as_datetime_oapg, .as_decimal_oapg, .as_uuid_oapg
    - type number + format: See .as_float_oapg, .as_int_oapg
    - this was done because openapi/json-schema defines constraints. string data may be type string with no format
    keyword in one schema, and include a format constraint in another schema
    - So if you need to access a string format based type, use as_date_oapg/as_datetime_oapg/as_decimal_oapg/as_uuid_oapg
    - So if you need to access a number format based type, use as_int_oapg/as_float_oapg
7. Property access on AnyType(type unset) or object(dict) schemas
    - Only required keys with valid python names are properties like .someProp and have type hints
    - All optional keys may not exist, so properties are not defined for them
    - One can access optional values with dict_instance['optionalProp'] and KeyError will be raised if it does not exist
    - Use get_item_oapg if you need a way to always get a value whether or not the key exists
        - If the key does not exist, schemas.unset is returned from calling dict_instance.get_item_oapg('optionalProp')
        - All required and optional keys have type hints for this method, and @typing.overload is used
        - A type hint is also generated for additionalProperties accessed using this method
    - So you will need to update you code to use some_instance['optionalProp'] to access optional property
    and additionalProperty values
8. The location of the api classes has changed
    - Api classes are located in your_package.apis.tags.some_api
    - This change was made to eliminate redundant code generation
    - Legacy generators generated the same endpoint twice if it had > 1 tag on it
    - This generator defines an endpoint in one class, then inherits that class to generate
      apis by tags and by paths
    - This change reduces code and allows quicker run time if you use the path apis
        - path apis are at your_package.apis.paths.some_path
    - Those apis will only load their needed models, which is less to load than all of the resources needed in a tag api
    - So you will need to update your import paths to the api classes

### Why are Oapg and _oapg used in class and method names?
Classes can have arbitrarily named properties set on them
Endpoints can have arbitrary operationId method names set
For those reasons, I use the prefix Oapg and _oapg to greatly reduce the likelihood of collisions
on protected + public classes/methods.
oapg stands for OpenApi Python Generator.

### Object property spec case
This was done because when payloads are ingested, they can be validated against N number of schemas.
If the input signature used a different property name then that has mutated the payload.
So SchemaA and SchemaB must both see the camelCase spec named variable.
Also it is possible to send in two properties, named camelCase and camel_case in the same payload.
That use case should be support so spec case is used.

### Parameter spec case
Parameters can be included in different locations including:
- query
- path
- header
- cookie

Any of those parameters could use the same parameter names, so if every parameter
was included as an endpoint parameter in a function signature, they would collide.
For that reason, each of those inputs have been separated out into separate typed dictionaries:
- query_params
- path_params
- header_params
- cookie_params

So when updating your code, you will need to pass endpoint parameters in using those
dictionaries.

### Endpoint responses
Endpoint responses have been enriched to now include more information.
Any response reom an endpoint will now include the following properties:
response: urllib3.HTTPResponse
body: typing.Union[Unset, Schema]
headers: typing.Union[Unset, TODO]
Note: response header deserialization has not yet been added


## Installation & Usage
### pip install

If the python package is hosted on a repository, you can install directly using:

```sh
pip install git+https://github.com/GIT_USER_ID/GIT_REPO_ID.git
```
(you may need to run `pip` with root permission: `sudo pip install git+https://github.com/GIT_USER_ID/GIT_REPO_ID.git`)

Then import the package:
```python
import collibra_importer
```

### Setuptools

Install via [Setuptools](http://pypi.python.org/pypi/setuptools).

```sh
python setup.py install --user
```
(or `sudo python setup.py install` to install the package for all users)

Then import the package:
```python
import collibra_importer
```

## Getting Started

Please follow the [installation procedure](#installation--usage) and then run the following:

```python

import time
import collibra_importer
from pprint import pprint
from collibra_importer.apis.tags import model_import_api
from collibra_importer.model.job import Job
from collibra_importer.model.paged_response_synchronization_info import PagedResponseSynchronizationInfo
# Defining the host is optional and defaults to http://localhost/rest/2.0
# See configuration.py for a list of all supported configuration parameters.
configuration = collibra_importer.Configuration(
    host = "http://localhost/rest/2.0"
)

# The client must configure the authentication and authorization parameters
# in accordance with the API server security policy.
# Examples for each auth method are provided below, use the example that
# satisfies your auth use case.

# Configure HTTP basic authorization: basicAuth
configuration = collibra_importer.Configuration(
    username = 'YOUR_USERNAME',
    password = 'YOUR_PASSWORD'
)

# Configure Bearer authorization (JWT): bearerAuth
configuration = collibra_importer.Configuration(
    access_token = 'YOUR_BEARER_TOKEN'
)

# Enter a context with an instance of the API client
with collibra_importer.ApiClient(configuration) as api_client:
    # Create an instance of the API class
    api_instance = model_import_api.ModelImportApi(api_client)
    synchronization_id = "synchronizationId_example" # str | The synchronization id of the operation.

    try:
        # Removes all cache entries corresponding to the provided synchronization id.
        api_instance.evict_synchronization_cache(synchronization_id)
    except collibra_importer.ApiException as e:
        print("Exception when calling ModelImportApi->evict_synchronization_cache: %s\n" % e)
```

## Documentation for API Endpoints

All URIs are relative to *http://localhost/rest/2.0*

Class | Method | HTTP request | Description
------------ | ------------- | ------------- | -------------
*ModelImportApi* | [**evict_synchronization_cache**](docs/apis/tags/ModelImportApi.md#evict_synchronization_cache) | **delete** /import/synchronize/{synchronizationId}/evict | Removes all cache entries corresponding to the provided synchronization id.
*ModelImportApi* | [**exists**](docs/apis/tags/ModelImportApi.md#exists) | **get** /import/synchronize/exists/{synchronizationId} | Checks whether given synchronization id already exists.
*ModelImportApi* | [**find_synchronization_infos**](docs/apis/tags/ModelImportApi.md#find_synchronization_infos) | **get** /import/synchronize | Returns synchronization information matching the given search criteria.
*ModelImportApi* | [**import_csv_in_job**](docs/apis/tags/ModelImportApi.md#import_csv_in_job) | **post** /import/csv-job | Starts import from the CSV file in job (asynchronously).
*ModelImportApi* | [**import_excel_in_job**](docs/apis/tags/ModelImportApi.md#import_excel_in_job) | **post** /import/excel-job | Starts import from the Excel file in job (asynchronously).
*ModelImportApi* | [**import_json_in_job**](docs/apis/tags/ModelImportApi.md#import_json_in_job) | **post** /import/json-job | Starts import from the JSON file in job (asynchronously).
*ModelImportApi* | [**remove_synchronization**](docs/apis/tags/ModelImportApi.md#remove_synchronization) | **delete** /import/synchronize/{synchronizationId} | Removes all information about synchronization process corresponding to provided synchronization id.
*ModelImportApi* | [**synchronize_batch_csv_in_job**](docs/apis/tags/ModelImportApi.md#synchronize_batch_csv_in_job) | **post** /import/synchronize/{synchronizationId}/batch/csv-job | Starts batch synchronization from the CSV file in job (asynchronously).
*ModelImportApi* | [**synchronize_batch_excel_in_job**](docs/apis/tags/ModelImportApi.md#synchronize_batch_excel_in_job) | **post** /import/synchronize/{synchronizationId}/batch/excel-job | Starts batch synchronization from the Excel file in job (asynchronously).
*ModelImportApi* | [**synchronize_batch_json_in_job**](docs/apis/tags/ModelImportApi.md#synchronize_batch_json_in_job) | **post** /import/synchronize/{synchronizationId}/batch/json-job | Starts batch synchronization from the JSON file in job (asynchronously).
*ModelImportApi* | [**synchronize_csv_in_job**](docs/apis/tags/ModelImportApi.md#synchronize_csv_in_job) | **post** /import/synchronize/{synchronizationId}/csv-job | Starts full synchronization from the CSV file in job (asynchronously).
*ModelImportApi* | [**synchronize_excel_in_job**](docs/apis/tags/ModelImportApi.md#synchronize_excel_in_job) | **post** /import/synchronize/{synchronizationId}/excel-job | Starts full synchronization from the Excel file in job (asynchronously).
*ModelImportApi* | [**synchronize_finalize_in_job**](docs/apis/tags/ModelImportApi.md#synchronize_finalize_in_job) | **post** /import/synchronize/{synchronizationId}/finalize/job | Starts synchronization finalization in job (asynchronously).
*ModelImportApi* | [**synchronize_json_in_job**](docs/apis/tags/ModelImportApi.md#synchronize_json_in_job) | **post** /import/synchronize/{synchronizationId}/json-job | Starts full synchronization from the JSON file in job (asynchronously).
*ImportResultsApi* | [**find_import_errors**](docs/apis/tags/ImportResultsApi.md#find_import_errors) | **get** /import/results/{jobId}/errors | List the errors of a finished import job
*ImportResultsApi* | [**get_import_job_summary**](docs/apis/tags/ImportResultsApi.md#get_import_job_summary) | **get** /import/results/{jobId}/summary | Retrieve a summary of a finished import job

## Documentation For Models

 - [AssetIdentifier](docs/models/AssetIdentifier.md)
 - [AssetImportCommand](docs/models/AssetImportCommand.md)
 - [AssetTypeIdentifier](docs/models/AssetTypeIdentifier.md)
 - [AttributeValue](docs/models/AttributeValue.md)
 - [CategoryReference](docs/models/CategoryReference.md)
 - [CommunityIdentifier](docs/models/CommunityIdentifier.md)
 - [CommunityImportCommand](docs/models/CommunityImportCommand.md)
 - [ComplexRelationIdentifier](docs/models/ComplexRelationIdentifier.md)
 - [ComplexRelationImportCommand](docs/models/ComplexRelationImportCommand.md)
 - [ComplexRelationTypeIdentifier](docs/models/ComplexRelationTypeIdentifier.md)
 - [DomainIdentifier](docs/models/DomainIdentifier.md)
 - [DomainImportCommand](docs/models/DomainImportCommand.md)
 - [DomainTypeIdentifier](docs/models/DomainTypeIdentifier.md)
 - [ExternalIdentifier](docs/models/ExternalIdentifier.md)
 - [FindSynchronizationRequest](docs/models/FindSynchronizationRequest.md)
 - [ImportCommandReference](docs/models/ImportCommandReference.md)
 - [ImportCounters](docs/models/ImportCounters.md)
 - [ImportCsvInJobRequest](docs/models/ImportCsvInJobRequest.md)
 - [ImportError](docs/models/ImportError.md)
 - [ImportErrorPagedResponse](docs/models/ImportErrorPagedResponse.md)
 - [ImportExcelInJobRequest](docs/models/ImportExcelInJobRequest.md)
 - [ImportJsonInJobRequest](docs/models/ImportJsonInJobRequest.md)
 - [ImportSummary](docs/models/ImportSummary.md)
 - [Job](docs/models/Job.md)
 - [MappingIdentifier](docs/models/MappingIdentifier.md)
 - [MappingImportCommand](docs/models/MappingImportCommand.md)
 - [Owner](docs/models/Owner.md)
 - [PagedResponseImportError](docs/models/PagedResponseImportError.md)
 - [PagedResponseSynchronizationInfo](docs/models/PagedResponseSynchronizationInfo.md)
 - [ResourceTypeSummary](docs/models/ResourceTypeSummary.md)
 - [StatusIdentifier](docs/models/StatusIdentifier.md)
 - [SubcategorySummary](docs/models/SubcategorySummary.md)
 - [SynchronizationBatchCsvInJobRequest](docs/models/SynchronizationBatchCsvInJobRequest.md)
 - [SynchronizationBatchExcelInJobRequest](docs/models/SynchronizationBatchExcelInJobRequest.md)
 - [SynchronizationBatchJsonInJobRequest](docs/models/SynchronizationBatchJsonInJobRequest.md)
 - [SynchronizationCsvInJobRequest](docs/models/SynchronizationCsvInJobRequest.md)
 - [SynchronizationExcelInJobRequest](docs/models/SynchronizationExcelInJobRequest.md)
 - [SynchronizationFinalizationRequest](docs/models/SynchronizationFinalizationRequest.md)
 - [SynchronizationInfo](docs/models/SynchronizationInfo.md)
 - [SynchronizationJsonInJobRequest](docs/models/SynchronizationJsonInJobRequest.md)
 - [UserGroupIdentifier](docs/models/UserGroupIdentifier.md)
 - [UserIdentifier](docs/models/UserIdentifier.md)

## Documentation For Authorization


## basicAuth

- **Type**: HTTP basic authentication

 Authentication schemes defined for the API:
## bearerAuth

- **Type**: Bearer authentication (JWT)


## Author




## Notes for Large OpenAPI documents
If the OpenAPI document is large, imports in collibra_importer.apis and collibra_importer.models may fail with a
RecursionError indicating the maximum recursion limit has been exceeded. In that case, there are a couple of solutions:

Solution 1:
Use specific imports for apis and models like:
- `from collibra_importer.apis.default_api import DefaultApi`
- `from collibra_importer.model.pet import Pet`

Solution 1:
Before importing the package, adjust the maximum recursion limit as shown below:
```
import sys
sys.setrecursionlimit(1500)
import collibra_importer
from collibra_importer.apis import *
from collibra_importer.models import *
```
